# app/api/chores/router.py
from typing import Annotated, List, Optional
from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.dependencies import check_house_membership, get_current_user
from app.db.session import get_db
from app.models.chore import Chore
from app.models.user import User
from app.schemas.chore import (
    Chore as ChoreSchema,
    ChoreAssign,
    ChoreComplete,
    ChoreCreate,
    ChoreUpdate,
)

router = APIRouter()


@router.get("/{house_id}/chores", response_model=List[ChoreSchema])
async def get_chores(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all chores for a house."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get all chores for the house
    result = await db.execute(select(Chore).where(Chore.house_id == house_id))
    chores = result.scalars().all()
    return chores


@router.post(
    "/{house_id}/chores", response_model=ChoreSchema, status_code=status.HTTP_201_CREATED
)
async def create_chore(
    house_id: UUID,
    chore_in: ChoreCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Create new chore for a house."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # If assigned_to is provided, check if the user is a member of the house
    if chore_in.assigned_to:
        is_assignee_member = await check_house_membership(
            db, str(chore_in.assigned_to), str(house_id)
        )
        if not is_assignee_member:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Assigned user is not a member of this house",
            )
    
    # Create the chore
    db_chore = Chore(
        house_id=house_id,
        **chore_in.model_dump(),
    )
    db.add(db_chore)
    await db.commit()
    await db.refresh(db_chore)
    return db_chore


@router.put("/{house_id}/chores/{chore_id}", response_model=ChoreSchema)
async def update_chore(
    house_id: UUID,
    chore_id: UUID,
    chore_in: ChoreUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update chore."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the chore
    result = await db.execute(
        select(Chore).where(
            Chore.id == chore_id,
            Chore.house_id == house_id,
        )
    )
    chore = result.scalars().first()
    
    if not chore:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Chore not found",
        )
    
    # If assigned_to is provided, check if the user is a member of the house
    if chore_in.assigned_to:
        is_assignee_member = await check_house_membership(
            db, str(chore_in.assigned_to), str(house_id)
        )
        if not is_assignee_member:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Assigned user is not a member of this house",
            )
    
    # Update chore fields
    update_data = chore_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(chore, field, value)
    
    await db.commit()
    await db.refresh(chore)
    return chore


@router.delete("/{house_id}/chores/{chore_id}", status_code=status.HTTP_204_NO_CONTENT)
async def delete_chore(
    house_id: UUID,
    chore_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Delete chore."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the chore
    result = await db.execute(
        select(Chore).where(
            Chore.id == chore_id,
            Chore.house_id == house_id,
        )
    )
    chore = result.scalars().first()
    
    if not chore:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Chore not found",
        )
    
    # Delete the chore
    await db.delete(chore)
    await db.commit()
    return None


@router.patch("/{house_id}/chores/{chore_id}/assign", response_model=ChoreSchema)
async def assign_chore(
    house_id: UUID,
    chore_id: UUID,
    assign_data: ChoreAssign,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Assign chore to user."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the chore
    result = await db.execute(
        select(Chore).where(
            Chore.id == chore_id,
            Chore.house_id == house_id,
        )
    )
    chore = result.scalars().first()
    
    if not chore:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Chore not found",
        )
    
    # Check if the assigned user is a member of the house
    is_assignee_member = await check_house_membership(
        db, str(assign_data.assigned_to), str(house_id)
    )
    if not is_assignee_member:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Assigned user is not a member of this house",
        )
    
    # Assign the chore
    chore.assigned_to = assign_data.assigned_to
    await db.commit()
    await db.refresh(chore)
    return chore


@router.patch("/{house_id}/chores/{chore_id}/complete", response_model=ChoreSchema)
async def complete_chore(
    house_id: UUID,
    chore_id: UUID,
    complete_data: ChoreComplete,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Mark chore as complete/incomplete."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the chore
    result = await db.execute(
        select(Chore).where(
            Chore.id == chore_id,
            Chore.house_id == house_id,
        )
    )
    chore = result.scalars().first()
    
    if not chore:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Chore not found",
        )
    
    # Update completion status
    chore.is_completed = complete_data.is_completed
    await db.commit()
    await db.refresh(chore)
    return chore
