# app/api/expenses/router.py
from decimal import Decimal
from typing import Annotated, Dict, List
from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.dependencies import check_house_membership, get_current_user
from app.db.session import get_db
from app.models.expense import Expense, ExpenseSplit
from app.models.house import HouseMember
from app.models.shopping_list import ShoppingList
from app.models.user import User
from app.schemas.expense import (
    Expense as ExpenseSchema,
    ExpenseCreate,
    ExpenseSummary,
    ExpenseUpdate,
)

router = APIRouter()


@router.get("/{house_id}/lists/{list_id}/expenses", response_model=List[ExpenseSchema])
async def get_expenses(
    house_id: UUID,
    list_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all expenses for a list."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get all expenses for the list
    result = await db.execute(select(Expense).where(Expense.list_id == list_id))
    expenses = result.scalars().all()
    return expenses


@router.post(
    "/{house_id}/lists/{list_id}/expenses",
    response_model=ExpenseSchema,
    status_code=status.HTTP_201_CREATED,
)
async def create_expense(
    house_id: UUID,
    list_id: UUID,
    expense_in: ExpenseCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Create expense."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Check if payer is a member of the house
    is_payer_member = await check_house_membership(
        db, str(expense_in.payer_id), str(house_id)
    )
    if not is_payer_member:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Payer is not a member of this house",
        )
    
    # Validate that the sum of splits equals the total amount
    splits_total = sum(split.amount for split in expense_in.splits)
    if splits_total != expense_in.amount:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Sum of splits must equal the total expense amount",
        )
    
    # Create the expense
    db_expense = Expense(
        list_id=list_id,
        payer_id=expense_in.payer_id,
        amount=expense_in.amount,
        description=expense_in.description,
        date=expense_in.date,
    )
    db.add(db_expense)
    await db.flush()
    
    # Create the splits
    for split in expense_in.splits:
        # Check if user is a member of the house
        is_user_member = await check_house_membership(
            db, str(split.user_id), str(house_id)
        )
        if not is_user_member:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=f"User {split.user_id} is not a member of this house",
            )
        
        db_split = ExpenseSplit(
            expense_id=db_expense.id,
            user_id=split.user_id,
            amount=split.amount,
        )
        db.add(db_split)
    
    await db.commit()
    await db.refresh(db_expense)
    return db_expense


@router.put("/{house_id}/lists/{list_id}/expenses/{expense_id}", response_model=ExpenseSchema)
async def update_expense(
    house_id: UUID,
    list_id: UUID,
    expense_id: UUID,
    expense_in: ExpenseUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update expense."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the expense
    result = await db.execute(
        select(Expense).where(
            Expense.id == expense_id,
            Expense.list_id == list_id,
        )
    )
    expense = result.scalars().first()
    
    if not expense:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Expense not found",
        )
    
    # Update expense fields
    update_data = expense_in.model_dump(exclude_unset=True)
    
    # Handle splits separately
    splits = update_data.pop("splits", None)
    
    # Update basic expense fields
    for field, value in update_data.items():
        if field == "payer_id" and value:
            # Check if new payer is a member of the house
            is_payer_member = await check_house_membership(
                db, str(value), str(house_id)
            )
            if not is_payer_member:
                raise HTTPException(
                    status_code=status.HTTP_400_BAD_REQUEST,
                    detail="Payer is not a member of this house",
                )
        setattr(expense, field, value)
    
    # Handle splits if provided
    if splits is not None:
        # Validate that the sum of splits equals the total amount
        splits_total = sum(split.amount for split in splits)
        if splits_total != expense.amount:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Sum of splits must equal the total expense amount",
            )
        
        # Delete existing splits
        await db.execute(
            "DELETE FROM expense_splits WHERE expense_id = :expense_id",
            {"expense_id": expense_id},
        )
        
        # Create new splits
        for split in splits:
            # Check if user is a member of the house
            is_user_member = await check_house_membership(
                db, str(split.user_id), str(house_id)
            )
            if not is_user_member:
                raise HTTPException(
                    status_code=status.HTTP_400_BAD_REQUEST,
                    detail=f"User {split.user_id} is not a member of this house",
                )
            
            db_split = ExpenseSplit(
                expense_id=expense_id,
                user_id=split.user_id,
                amount=split.amount,
            )
            db.add(db_split)
    
    await db.commit()
    await db.refresh(expense)
    return expense


@router.get("/{house_id}/lists/{list_id}/expenses/summary", response_model=ExpenseSummary)
async def get_expense_summary(
    house_id: UUID,
    list_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get expense summary and splits."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get all expenses for the list
    result = await db.execute(select(Expense).where(Expense.list_id == list_id))
    expenses = result.scalars().all()
    
    # Get all house members
    result = await db.execute(
        select(HouseMember).where(HouseMember.house_id == house_id)
    )
    members = result.scalars().all()
    
    # Calculate total amount
    total_amount = sum(expense.amount for expense in expenses)
    
    # Calculate user balances
    user_balances: Dict[UUID, Decimal] = {member.user_id: Decimal("0") for member in members}
    
    # For each expense, add the amount paid to the payer's balance
    # and subtract the split amounts from each user's balance
    for expense in expenses:
        # Add the full amount to the payer's balance (they paid this amount)
        user_balances[expense.payer_id] += expense.amount
        
        # Subtract each user's split from their balance (they owe this amount)
        for split in expense.splits:
            user_balances[split.user_id] -= split.amount
    
    return ExpenseSummary(
        total_amount=total_amount,
        user_balances={user_id: balance for user_id, balance in user_balances.items()},
    )
