from datetime import datetime, timedelta
from sqlalchemy.future import select
from app.models.invite import HouseInvite
from app.schemas.invite import HouseInvite as HouseInviteSchema, HouseInviteCreate
from app.services.invite_service import generate_invite_code

from typing import Annotated, List
from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.dependencies import check_house_membership, get_current_user
from app.db.session import get_db
from app.models.house import House, HouseMember
from app.models.user import User
from app.schemas.house import (
    House as HouseSchema,
    HouseCreate,
    HouseMember as HouseMemberSchema,
    HouseMemberCreate,
    HouseMemberUpdate,
    HouseUpdate,
    HouseMemberResponse,
)

router = APIRouter()


@router.get("/", response_model=List[HouseSchema])
async def get_houses(
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all houses for the authenticated user."""
    query = (
        select(House)
        .join(HouseMember, House.id == HouseMember.house_id)
        .where(HouseMember.user_id == current_user.id)
    )
    result = await db.execute(query)
    houses = result.scalars().all()
    return houses


@router.post("/", response_model=HouseSchema, status_code=status.HTTP_201_CREATED)
async def create_house(
    house_in: HouseCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Create a new house."""
    # Create the house
    db_house = House(**house_in.model_dump())
    db.add(db_house)
    await db.flush()
    
    # Add the current user as an admin
    db_member = HouseMember(
        house_id=db_house.id,
        user_id=current_user.id,
        role="admin",
    )
    db.add(db_member)
    
    await db.commit()
    await db.refresh(db_house)
    return db_house


@router.get("/{house_id}", response_model=HouseSchema)
async def get_house(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get house details."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the house
    result = await db.execute(select(House).where(House.id == house_id))
    house = result.scalars().first()
    
    if not house:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="House not found",
        )
    
    return house


@router.put("/{house_id}", response_model=HouseSchema)
async def update_house(
    house_id: UUID,
    house_in: HouseUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update house details."""
    # Check if user is an admin of the house
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Must be an admin to update house details",
        )
    
    # Get the house
    result = await db.execute(select(House).where(House.id == house_id))
    house = result.scalars().first()
    
    if not house:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="House not found",
        )
    
    # Update house fields
    update_data = house_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(house, field, value)
    
    await db.commit()
    await db.refresh(house)
    return house


@router.delete("/{house_id}", status_code=status.HTTP_204_NO_CONTENT)
async def delete_house(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Delete house."""
    # Check if user is an admin of the house
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Must be an admin to delete house",
        )
    
    # Get the house
    result = await db.execute(select(House).where(House.id == house_id))
    house = result.scalars().first()
    
    if not house:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="House not found",
        )
    
    # Delete the house
    await db.delete(house)
    await db.commit()
    return None


# House Members endpoints
@router.get("/{house_id}/members", response_model=List[HouseMemberSchema])
async def get_house_members(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all members of a house."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get all members
    result = await db.execute(
        select(HouseMember).where(HouseMember.house_id == house_id)
    )
    members = result.scalars().all()
    return members


@router.post(
    "/{house_id}/members", response_model=HouseMemberResponse, status_code=status.HTTP_201_CREATED
)
async def add_house_member(
    house_id: UUID,
    member_in: HouseMemberCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Add a member to the house."""
    # Check if user is an admin of the house
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Must be an admin to add members",
        )
    
    # Check if user exists
    result = await db.execute(select(User).where(User.id == member_in.user_id))
    user = result.scalars().first()
    if not user:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="User not found",
        )
    
    # Check if user is already a member
    result = await db.execute(
        select(HouseMember).where(
            HouseMember.house_id == house_id,
            HouseMember.user_id == member_in.user_id,
        )
    )
    existing_member = result.scalars().first()
    if existing_member:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="User is already a member of this house",
        )
    
    # Add the member
    db_member = HouseMember(
        house_id=house_id,
        user_id=member_in.user_id,
        role=member_in.role,
    )
    db.add(db_member)
    await db.commit()
    await db.refresh(db_member)
    return db_member


@router.put("/{house_id}/members/{user_id}", response_model=HouseMemberSchema)
async def update_house_member(
    house_id: UUID,
    user_id: UUID,
    member_in: HouseMemberUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update member role."""
    # Check if user is an admin of the house
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Must be an admin to update member roles",
        )
    
    # Get the member
    result = await db.execute(
        select(HouseMember).where(
            HouseMember.house_id == house_id,
            HouseMember.user_id == user_id,
        )
    )
    member = result.scalars().first()
    
    if not member:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Member not found",
        )
    
    # Update member fields
    update_data = member_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(member, field, value)
    
    await db.commit()
    await db.refresh(member)
    return member


@router.delete("/{house_id}/members/{user_id}", status_code=status.HTTP_204_NO_CONTENT)
async def remove_house_member(
    house_id: UUID,
    user_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Remove a member from the house."""
    # Check if user is an admin of the house
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Must be an admin to remove members",
        )
    
    # Get the member
    result = await db.execute(
        select(HouseMember).where(
            HouseMember.house_id == house_id,
            HouseMember.user_id == user_id,
        )
    )
    member = result.scalars().first()
    
    if not member:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Member not found",
        )
    
    # Prevent removing the last admin
    if member.role == "admin":
        # Count admins
        result = await db.execute(
            select(HouseMember).where(
                HouseMember.house_id == house_id,
                HouseMember.role == "admin",
            )
        )
        admins = result.scalars().all()
        if len(admins) <= 1:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Cannot remove the last admin of the house",
            )
    
    # Remove the member
    await db.delete(member)
    await db.commit()
    return None

# Create a new invite (only admins can create invites)
@router.post(
    "/{house_id}/invites",
    response_model=HouseInviteSchema,
    status_code=status.HTTP_201_CREATED,
)
async def create_house_invite(
    house_id: UUID,
    invite_in: HouseInviteCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """
    Create an invitation code for a house with a set expiration.
    Only an admin can create an invitation.
    """
    is_admin = await check_house_membership(
        db, str(current_user.id), str(house_id), required_role="admin"
    )
    if not is_admin:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Only admins can create invitation codes",
        )

    # Generate a unique code
    code = generate_invite_code()
    # Check for uniqueness (in case of a rare collision)
    while True:
        result = await db.execute(select(HouseInvite).where(HouseInvite.code == code))
        existing_invite = result.scalars().first()
        if existing_invite:
            code = generate_invite_code()
        else:
            break

    expires_at = datetime.utcnow() + timedelta(minutes=invite_in.expires_in_minutes)
    db_invite = HouseInvite(
        code=code,
        house_id=house_id,
        inviter_id=current_user.id,
        expires_at=expires_at,
    )
    db.add(db_invite)
    await db.commit()
    await db.refresh(db_invite)
    return db_invite


# List active (nonexpired) invites for a house (accessible to house members)
@router.get(
    "/{house_id}/invites",
    response_model=list[HouseInviteSchema],
)
async def list_active_invites(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """
    Get all nonexpired invitation codes for a given house.
    """
    is_member = await check_house_membership(
        db, str(current_user.id), str(house_id)
    )
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )

    invites_query = select(HouseInvite).where(
        HouseInvite.house_id == house_id,
        HouseInvite.expires_at > datetime.utcnow(),
    )
    result = await db.execute(invites_query)
    invites = result.scalars().all()
    return invites


# Accept an invite code (allows a user to join a house)
@router.post(
    "/invites/accept",
    response_model=HouseMemberResponse,
    status_code=status.HTTP_201_CREATED,
)
async def accept_invite(
    code: str,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """
    Accept an invitation code to join a house.
    If the invite has expired or is invalid, an error is raised.
    """
    result = await db.execute(select(HouseInvite).where(HouseInvite.code == code))
    invite = result.scalars().first()
    if not invite:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Invalid invitation code",
        )
    if invite.expires_at < datetime.utcnow():
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Invitation code has expired",
        )

    # Check if the user is already a member
    result = await db.execute(
        select(HouseMember).where(
            HouseMember.house_id == invite.house_id,
            HouseMember.user_id == current_user.id,
        )
    )
    existing_member = result.scalars().first()
    if existing_member:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="User is already a member of this house",
        )

    db_member = HouseMember(
        house_id=invite.house_id,
        user_id=current_user.id,
        role="member",  # default role on joining
    )
    db.add(db_member)
    await db.commit()
    await db.refresh(db_member)
    return db_member