# app/api/ocr/router.py
from typing import Annotated, List
from uuid import UUID

from fastapi import APIRouter, Depends, File, HTTPException, UploadFile, status
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.dependencies import check_house_membership, get_current_user
from app.db.session import get_db
from app.models.shopping_list import ListItem, ShoppingList
from app.models.user import User
from app.schemas.shopping_list import ListItemCreate
from app.services.ocr_service import ocr_service

router = APIRouter()


@router.post("/process", response_model=List[dict])
async def process_image(
    current_user: Annotated[User, Depends(get_current_user)],
    image: UploadFile = File(...),
):
    """Process image and extract items."""
    if not image.content_type.startswith("image/"):
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="File must be an image",
        )
    
    # Read the image file
    image_bytes = await image.read()
    
    # Process the image with OCR
    items = await ocr_service.process_image(image_bytes)
    
    return items


@router.post("/{house_id}/lists/{list_id}/ocr/apply")
async def apply_ocr_results(
    house_id: UUID,
    list_id: UUID,
    items: List[dict],
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Add OCR results to shopping list."""
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the highest position to place new items at the end
    result = await db.execute(
        select(ListItem).where(ListItem.list_id == list_id).order_by(ListItem.position.desc())
    )
    last_item = result.scalars().first()
    position = 1 if not last_item else (last_item.position or 0) + 1
    
    # Add items to the list
    added_items = []
    for item_data in items:
        # Create item schema for validation
        try:
            item_create = ListItemCreate(
                name=item_data.get("name", "Unknown Item"),
                quantity=item_data.get("quantity", 1),
                price=item_data.get("price"),
                unit=item_data.get("unit"),
            )
            
            # Create the item
            db_item = ListItem(
                list_id=list_id,
                position=position,
                **item_create.model_dump(),
            )
            db.add(db_item)
            position += 1
            added_items.append(item_data)
        except Exception as e:
            # Log the error but continue with other items
            continue
    
    await db.commit()
    
    return {
        "detail": f"Added {len(added_items)} items to the shopping list",
        "items": added_items,
    }
