# app/api/shopping_lists/router.py
from typing import Annotated, List
from uuid import UUID

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from app.core.dependencies import check_house_membership, get_current_user
from app.db.session import get_db
from app.models.shopping_list import ListItem, ShoppingList
from app.models.user import User
from app.schemas.shopping_list import (
    ItemReorder,
    ListItem as ListItemSchema,
    ListItemCreate,
    ListItemUpdate,
    ShoppingList as ShoppingListSchema,
    ShoppingListCreate,
    ShoppingListUpdate,
)
from app.core.logger import shopping_lists_logger

router = APIRouter()


@router.get("/{house_id}/lists", response_model=List[ShoppingListSchema])
async def get_shopping_lists(
    house_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all shopping lists for a house."""
    shopping_lists_logger.debug(f"User {current_user.id} requested shopping lists for house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get all shopping lists for the house
    result = await db.execute(
        select(ShoppingList).where(ShoppingList.house_id == house_id)
    )
    lists = result.scalars().all()
    return lists


@router.post(
    "/{house_id}/lists", response_model=ShoppingListSchema, status_code=status.HTTP_201_CREATED
)
async def create_shopping_list(
    house_id: UUID,
    list_in: ShoppingListCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Create new shopping list for a house."""
    shopping_lists_logger.debug(f"User {current_user.id} is creating a new shopping list for house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Create the shopping list
    db_list = ShoppingList(
        house_id=house_id,
        **list_in.model_dump(),
    )
    db.add(db_list)
    await db.commit()
    await db.refresh(db_list)
    return db_list


@router.get("/{house_id}/lists/{list_id}", response_model=ShoppingListSchema)
async def get_shopping_list(
    house_id: UUID,
    list_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get single list details."""
    shopping_lists_logger.debug(f"User {current_user.id} requested details for list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the shopping list
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    return shopping_list


@router.put("/{house_id}/lists/{list_id}", response_model=ShoppingListSchema)
async def update_shopping_list(
    house_id: UUID,
    list_id: UUID,
    list_in: ShoppingListUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update list details."""
    shopping_lists_logger.debug(f"User {current_user.id} is updating list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the shopping list
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Update shopping list fields
    update_data = list_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(shopping_list, field, value)
    
    await db.commit()
    await db.refresh(shopping_list)
    return shopping_list


@router.delete("/{house_id}/lists/{list_id}", status_code=status.HTTP_204_NO_CONTENT)
async def delete_shopping_list(
    house_id: UUID,
    list_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Delete/archive list."""
    shopping_lists_logger.debug(f"User {current_user.id} is deleting list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Get the shopping list
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Archive the list instead of deleting
    shopping_list.is_archived = True
    await db.commit()
    return None


# List Items endpoints
@router.get("/{house_id}/lists/{list_id}/items", response_model=List[ListItemSchema])
async def get_list_items(
    house_id: UUID,
    list_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Get all items in a list."""
    shopping_lists_logger.debug(f"User {current_user.id} requested items for list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get all items in the list
    result = await db.execute(select(ListItem).where(ListItem.list_id == list_id))
    items = result.scalars().all()
    return items


@router.post(
    "/{house_id}/lists/{list_id}/items",
    response_model=ListItemSchema,
    status_code=status.HTTP_201_CREATED,
)
async def add_list_item(
    house_id: UUID,
    list_id: UUID,
    item_in: ListItemCreate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Add item to list."""
    shopping_lists_logger.debug(f"User {current_user.id} is adding an item to list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the highest position to place the new item at the end
    result = await db.execute(
        select(ListItem).where(ListItem.list_id == list_id).order_by(ListItem.position.desc())
    )
    last_item = result.scalars().first()
    new_position = 1 if not last_item else (last_item.position or 0) + 1
    
    # Create the item
    db_item = ListItem(
        list_id=list_id,
        position=new_position,
        **item_in.model_dump(),
    )
    db.add(db_item)
    await db.commit()
    await db.refresh(db_item)
    return db_item


@router.put("/{house_id}/lists/{list_id}/items/{item_id}", response_model=ListItemSchema)
async def update_list_item(
    house_id: UUID,
    list_id: UUID,
    item_id: UUID,
    item_in: ListItemUpdate,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Update item."""
    shopping_lists_logger.debug(f"User {current_user.id} is updating item {item_id} in list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the item
    result = await db.execute(
        select(ListItem).where(
            ListItem.id == item_id,
            ListItem.list_id == list_id,
        )
    )
    item = result.scalars().first()
    
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    
    # Update item fields
    update_data = item_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(item, field, value)
    
    await db.commit()
    await db.refresh(item)
    return item


@router.delete(
    "/{house_id}/lists/{list_id}/items/{item_id}", status_code=status.HTTP_204_NO_CONTENT
)
async def delete_list_item(
    house_id: UUID,
    list_id: UUID,
    item_id: UUID,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Delete item."""
    shopping_lists_logger.debug(f"User {current_user.id} is deleting item {item_id} in list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the item
    result = await db.execute(
        select(ListItem).where(
            ListItem.id == item_id,
            ListItem.list_id == list_id,
        )
    )
    item = result.scalars().first()
    
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    
    # Delete the item
    await db.delete(item)
    await db.commit()
    return None


@router.patch(
    "/{house_id}/lists/{list_id}/items/{item_id}/complete", response_model=ListItemSchema
)
async def mark_item_complete(
    house_id: UUID,
    list_id: UUID,
    item_id: UUID,
    is_completed: bool,
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Mark item as complete/incomplete."""
    shopping_lists_logger.debug(f"User {current_user.id} is marking item {item_id} as {'complete' if is_completed else 'incomplete'} in list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Get the item
    result = await db.execute(
        select(ListItem).where(
            ListItem.id == item_id,
            ListItem.list_id == list_id,
        )
    )
    item = result.scalars().first()
    
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    
    # Update completion status
    item.is_completed = is_completed
    await db.commit()
    await db.refresh(item)
    return item


@router.post("/{house_id}/lists/{list_id}/items/reorder")
async def reorder_items(
    house_id: UUID,
    list_id: UUID,
    reorder_data: List[ItemReorder],
    db: Annotated[AsyncSession, Depends(get_db)],
    current_user: Annotated[User, Depends(get_current_user)],
):
    """Reorder items in list."""
    shopping_lists_logger.debug(f"User {current_user.id} is reordering items in list {list_id} in house {house_id}")
    # Check if user is a member of the house
    is_member = await check_house_membership(db, str(current_user.id), str(house_id))
    if not is_member:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="Not a member of this house",
        )
    
    # Check if the list belongs to the house
    result = await db.execute(
        select(ShoppingList).where(
            ShoppingList.id == list_id,
            ShoppingList.house_id == house_id,
        )
    )
    shopping_list = result.scalars().first()
    
    if not shopping_list:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Shopping list not found",
        )
    
    # Update positions for each item
    for item_order in reorder_data:
        result = await db.execute(
            select(ListItem).where(
                ListItem.id == item_order.item_id,
                ListItem.list_id == list_id,
            )
        )
        item = result.scalars().first()
        
        if item:
            item.position = item_order.new_position
    
    await db.commit()
    return {"detail": "Items reordered successfully"}
