# app/api/users/router.py
from typing import Annotated

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.ext.asyncio import AsyncSession

from app.db.session import get_db
from app.core.dependencies import get_current_user
from app.models.user import User as UserModel
from app.schemas.user import User, UserUpdate
from app.core.security import get_password_hash
from app.core.logger import logger

router = APIRouter()


@router.get("/me", response_model=User)
async def read_current_user(
    current_user: Annotated[UserModel, Depends(get_current_user)]
):
    """
    Retrieve the current user's profile.
    """
    logger.info(f"User {current_user.id} profile retrieved")
    return current_user


@router.put("/me", response_model=User)
async def update_current_user(
    user_in: UserUpdate,
    current_user: Annotated[UserModel, Depends(get_current_user)],
    db: Annotated[AsyncSession, Depends(get_db)],
):
    """
    Update the current user’s profile.
    If a password is provided, it will be hashed before also updating.
    """
    update_data = user_in.model_dump(exclude_unset=True)

    # If password is provided in the update, hash it first.
    if "password" in update_data:
        update_data["password_hash"] = get_password_hash(update_data.pop("password"))

    # Update each field on the current user
    for field, value in update_data.items():
        setattr(current_user, field, value)

    db.add(current_user)
    await db.commit()
    await db.refresh(current_user)
    logger.info(f"User {current_user.id} profile updated")
    return current_user
