# app/core/dependencies.py
from typing import Annotated, Optional

from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer
from jose import JWTError, jwt
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select

from app.core.config import settings
from app.core.security import ALGORITHM
from app.db.session import get_db
from app.models.user import User

oauth2_scheme = OAuth2PasswordBearer(tokenUrl=f"{settings.API_V1_STR}/auth/login")


async def get_current_user(
    db: Annotated[AsyncSession, Depends(get_db)],
    token: Annotated[str, Depends(oauth2_scheme)],
) -> User:
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="Could not validate credentials",
        headers={"WWW-Authenticate": "Bearer"},
    )
    try:
        payload = jwt.decode(token, settings.SECRET_KEY, algorithms=[ALGORITHM])
        user_id: str = payload.get("sub")
        if user_id is None:
            raise credentials_exception
    except JWTError:
        raise credentials_exception
    
    result = await db.execute(select(User).where(User.id == user_id))
    user = result.scalars().first()
    
    if user is None:
        raise credentials_exception
    return user


async def check_house_membership(
    db: AsyncSession, user_id: str, house_id: str, required_role: Optional[str] = None
) -> bool:
    """Check if a user is a member of a house with optional role check."""
    from app.models.house import HouseMember
    
    query = select(HouseMember).where(
        HouseMember.user_id == user_id,
        HouseMember.house_id == house_id
    )
    
    result = await db.execute(query)
    membership = result.scalars().first()
    
    if not membership:
        return False
    
    if required_role and membership.role != required_role:
        return False
        
    return True
