# app/main.py
from fastapi import FastAPI, Request
from fastapi.middleware.cors import CORSMiddleware
from app.core.logger import logger 

from app.api.auth.router import router as auth_router
from app.api.users.router import router as users_router
from app.api.chores.router import router as chores_router
from app.api.expenses.router import router as expenses_router
from app.api.houses.router import router as houses_router
from app.api.ocr.router import router as ocr_router
from app.api.shopping_lists.router import router as shopping_lists_router
from app.core.config import settings

app = FastAPI(
    title=settings.PROJECT_NAME,
    openapi_url=f"{settings.API_V1_STR}/openapi.json",
)

# Set all CORS enabled origins
if settings.BACKEND_CORS_ORIGINS:
    app.add_middleware(
        CORSMiddleware,
        allow_origins=[str(origin) for origin in settings.BACKEND_CORS_ORIGINS],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

@app.middleware("http")
async def log_requests(request: Request, call_next):
    logger.info(f"Request: {request.method} {request.url}")
    response = await call_next(request)
    logger.info(f"Response: {response.status_code}")
    return response

@app.on_event("startup")
async def startup_event():
    logger.info("Starting up...")

@app.on_event("shutdown")
async def shutdown_event():
    logger.info("Shutting down...")

# Include routers
app.include_router(auth_router, prefix=f"{settings.API_V1_STR}/auth", tags=["auth"])
app.include_router(houses_router, prefix=f"{settings.API_V1_STR}/houses", tags=["houses"])
app.include_router(
    shopping_lists_router, prefix=f"{settings.API_V1_STR}/houses", tags=["shopping_lists"]
)
app.include_router(
    expenses_router, prefix=f"{settings.API_V1_STR}/houses", tags=["expenses"]
)
app.include_router(
    chores_router, prefix=f"{settings.API_V1_STR}/houses", tags=["chores"]
)
app.include_router(ocr_router, prefix=f"{settings.API_V1_STR}/ocr", tags=["ocr"])
app.include_router(
    users_router,
    prefix=f"{settings.API_V1_STR}/users",
    tags=["users"]
)

@app.get("/")
async def root():
    return {"message": "Welcome to the Household API"}
