# app/services/ocr_service.py
import io
import logging
from typing import List, Optional

# from google.cloud import vision
# from google.cloud.vision_v1 import types

from app.core.config import settings
from app.schemas.shopping_list import ListItemCreate

logger = logging.getLogger(__name__)


class OCRService:
    def __init__(self):
        self.client = None
        if settings.OCR_API_KEY:
            try:
                self.client = vision.ImageAnnotatorClient.from_service_account_json(
                    settings.OCR_API_KEY
                )
            except Exception as e:
                logger.error(f"Failed to initialize OCR client: {e}")

    async def process_image(self, image_bytes: bytes) -> List[dict]:
        """Process an image and extract text."""
        if not self.client:
            logger.error("OCR client not initialized")
            return []

        try:
            image = types.Image(content=image_bytes)
            response = self.client.text_detection(image=image)
            texts = response.text_annotations

            # The first text annotation contains the entire detected text
            if not texts:
                return []

            # Extract potential shopping items from the text
            full_text = texts[0].description
            return self._extract_items_from_text(full_text)
        except Exception as e:
            logger.error(f"Error processing image: {e}")
            return []

    def _extract_items_from_text(self, text: str) -> List[dict]:
        """Extract potential shopping items from the OCR text."""
        # This is a simplified implementation
        # In a real-world scenario, you would use more sophisticated NLP techniques
        
        lines = text.split("\n")
        items = []
        
        for line in lines:
            # Skip very short lines or lines that are likely headers
            if len(line) < 3 or line.isupper() or "TOTAL" in line.upper():
                continue
                
            # Try to extract item name and potentially price
            parts = line.split()
            if not parts:
                continue
                
            # Simple heuristic: look for price patterns
            item_name = " ".join(parts[:-1]) if len(parts) > 1 else line
            price = None
            
            # Check if the last part looks like a price
            if len(parts) > 1 and parts[-1].replace(".", "").replace(",", "").isdigit():
                try:
                    price = float(parts[-1].replace(",", "."))
                    item_name = " ".join(parts[:-1])
                except ValueError:
                    pass
            
            # Add the item if we have a name
            if item_name.strip():
                items.append({
                    "name": item_name.strip(),
                    "price": price,
                    "quantity": 1,
                })
        
        return items


ocr_service = OCRService()
