import type {
    User,
    House,
    HouseMember,
    ShoppingList,
    ListItem,
    Expense,
    ExpenseSummary,
    Chore,
    HouseInvite
} from './types';

const API_URL = 'http://localhost:8000/api';
// import.meta.env.VITE_API_URL ||
async function handleResponse<T>(response: Response): Promise<T> {
    if (!response.ok) {
        const errorData = await response.json().catch(() => ({}));
        throw new Error(errorData.detail || `API error: ${response.status}`);
    }
    return response.json() as Promise<T>;
}

export const authApi = {
    async register(email: string, password: string, fullName?: string): Promise<User> {
        const response = await fetch(`${API_URL}/auth/register`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({
                email,
                password,
                full_name: fullName,
            }),
        });
        return handleResponse<User>(response);
    },

    async login(email: string, password: string): Promise<{ access_token: string; token_type: string }> {
        const formData = new FormData();
        formData.append('username', email);
        formData.append('password', password);

        const response = await fetch(`${API_URL}/auth/login`, {
            method: 'POST',
            body: formData,
        });
        return handleResponse<{ access_token: string; token_type: string }>(response);
    },

    async logout(): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/auth/logout`, {
            method: 'POST',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },

    isAuthenticated(): boolean {
        const token = localStorage.getItem('token');
        if (token) {
            return true
        }
        return false
    }
};

export const usersApi = {
    /**
     * Retrieves the profile information for the current user.
     * Requires a valid JWT stored in localStorage as "token".
     */
    async getProfile(): Promise<User> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/users/me`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<User>(response);
    },

    /**
     * Updates the current user's profile.
     * Accepts an object with fields such as full_name, email, or password.
     * Note: When updating the password, it will be hashed on the server side.
     */
    async updateProfile(data: {
        full_name?: string;
        email?: string;
        password?: string;
    }): Promise<User> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/users/me`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<User>(response);
    },
};

export const invitesApi = {
    /**
     * Creates a new invite for a house.
     * Only admins can create invites.
     */
    async createInvite(houseId: string, expiresInMinutes: number = 60): Promise<HouseInvite> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/invites`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({ expires_in_minutes: expiresInMinutes }),
        });
        return handleResponse<HouseInvite>(response);
    },

    /**
     * Lists all active (nonexpired) invites for a house.
     * Only house members can view invites.
     */
    async listActiveInvites(houseId: string): Promise<HouseInvite[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/invites`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<HouseInvite[]>(response);
    },

    /**
     * Accepts an invite code to join a house.
     * Any authenticated user can accept an invite (if it’s valid and not expired).
     */
    async acceptInvite(code: string): Promise<HouseMember> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/invites/accept?code=${code}`, {
            method: 'POST',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<HouseMember>(response);
    },
};

export const housesApi = {
    async getHouses(): Promise<House[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<House[]>(response);
    },

    async createHouse(name: string, description?: string): Promise<House> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                name,
                description,
            }),
        });
        return handleResponse<House>(response);
    },

    async getHouse(houseId: string): Promise<House> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<House>(response);
    },

    async updateHouse(houseId: string, data: { name?: string; description?: string }): Promise<House> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<House>(response);
    },

    async deleteHouse(houseId: string): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}`, {
            method: 'DELETE',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },

    // House Members
    async getHouseMembers(houseId: string): Promise<HouseMember[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/members`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<HouseMember[]>(response);
    },

    async addHouseMember(houseId: string, userId: string, role: string = 'member'): Promise<HouseMember> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/members`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                user_id: userId,
                role,
            }),
        });
        return handleResponse<HouseMember>(response);
    },

    async updateHouseMember(houseId: string, userId: string, role: string): Promise<HouseMember> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/members/${userId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                role,
            }),
        });
        return handleResponse<HouseMember>(response);
    },

    async removeHouseMember(houseId: string, userId: string): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/members/${userId}`, {
            method: 'DELETE',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },
};

export const shoppingListsApi = {
    async getLists(houseId: string): Promise<ShoppingList[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<ShoppingList[]>(response);
    },

    async createList(houseId: string, title: string, description?: string): Promise<ShoppingList> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                title,
                description,
            }),
        });
        return handleResponse<ShoppingList>(response);
    },

    async getList(houseId: string, listId: string): Promise<ShoppingList> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<ShoppingList>(response);
    },

    async updateList(
        houseId: string,
        listId: string,
        data: { title?: string; description?: string; is_archived?: boolean }
    ): Promise<ShoppingList> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<ShoppingList>(response);
    },

    async deleteList(houseId: string, listId: string): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}`, {
            method: 'DELETE',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },

    // List Items
    async getListItems(houseId: string, listId: string): Promise<ListItem[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/items`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<ListItem[]>(response);
    },

    async addListItem(
        houseId: string,
        listId: string,
        item: { name: string; quantity?: number; unit?: string; price?: number }
    ): Promise<ListItem> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/items`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(item),
        });
        return handleResponse<ListItem>(response);
    },

    async updateListItem(
        houseId: string,
        listId: string,
        itemId: string,
        data: { name?: string; quantity?: number; unit?: string; price?: number; is_completed?: boolean }
    ): Promise<ListItem> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/items/${itemId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<ListItem>(response);
    },

    async deleteListItem(houseId: string, listId: string, itemId: string): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/items/${itemId}`, {
            method: 'DELETE',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },

    async markItemComplete(
        houseId: string,
        listId: string,
        itemId: string,
        isCompleted: boolean
    ): Promise<ListItem> {
        const token = localStorage.getItem('token');
        const response = await fetch(
            `${API_URL}/houses/${houseId}/lists/${listId}/items/${itemId}/complete?is_completed=${isCompleted}`,
            {
                method: 'PATCH',
                headers: {
                    Authorization: `Bearer ${token}`,
                },
            }
        );
        return handleResponse<ListItem>(response);
    },

    async reorderItems(
        houseId: string,
        listId: string,
        reorderData: Array<{ item_id: string; new_position: number }>
    ): Promise<{ detail: string }> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/items/reorder`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(reorderData),
        });
        return handleResponse<{ detail: string }>(response);
    },
};

export const expensesApi = {
    async getExpenses(houseId: string, listId: string): Promise<Expense[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/expenses`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<Expense[]>(response);
    },

    async createExpense(
        houseId: string,
        listId: string,
        expense: {
            amount: number;
            payer_id: string;
            description?: string;
            date?: string;
            splits: Array<{ user_id: string; amount: number }>;
        }
    ): Promise<Expense> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/expenses`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(expense),
        });
        return handleResponse<Expense>(response);
    },

    async updateExpense(
        houseId: string,
        listId: string,
        expenseId: string,
        data: {
            amount?: number;
            payer_id?: string;
            description?: string;
            date?: string;
            splits?: Array<{ user_id: string; amount: number }>;
        }
    ): Promise<Expense> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/expenses/${expenseId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<Expense>(response);
    },

    async getExpenseSummary(houseId: string, listId: string): Promise<ExpenseSummary> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/expenses/summary`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<ExpenseSummary>(response);
    },
};

export const choresApi = {
    async getChores(houseId: string): Promise<Chore[]> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/chores`, {
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<Chore[]>(response);
    },

    async createChore(
        houseId: string,
        chore: {
            title: string;
            frequency: string;
            description?: string;
            assigned_to?: string;
            due_date?: string;
        }
    ): Promise<Chore> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses${houseId}/chores`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(chore),
        });
        return handleResponse<Chore>(response);
    },

    async updateChore(
        houseId: string,
        choreId: string,
        data: {
            title?: string;
            description?: string;
            assigned_to?: string;
            due_date?: string;
            is_completed?: boolean;
        }
    ): Promise<Chore> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/chores/${choreId}`, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(data),
        });
        return handleResponse<Chore>(response);
    },

    async deleteChore(houseId: string, choreId: string): Promise<void> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/chores/${choreId}`, {
            method: 'DELETE',
            headers: {
                Authorization: `Bearer ${token}`,
            },
        });
        return handleResponse<void>(response);
    },

    async assignChore(houseId: string, choreId: string, userId: string): Promise<Chore> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/chores/${choreId}/assign`, {
            method: 'PATCH',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                assigned_to: userId,
            }),
        });
        return handleResponse<Chore>(response);
    },

    async completeChore(houseId: string, choreId: string, isCompleted: boolean): Promise<Chore> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/chores/${choreId}/complete`, {
            method: 'PATCH',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify({
                is_completed: isCompleted,
            }),
        });
        return handleResponse<Chore>(response);
    },
};

export const ocrApi = {
    async processImage(imageFile: File): Promise<Array<{ name: string; price?: number; quantity?: number; unit?: string }>> {
        const token = localStorage.getItem('token');
        const formData = new FormData();
        formData.append('image', imageFile);

        const response = await fetch(`${API_URL}/ocr/process`, {
            method: 'POST',
            headers: {
                Authorization: `Bearer ${token}`,
            },
            body: formData,
        });
        return handleResponse<Array<{ name: string; price?: number; quantity?: number; unit?: string }>>(response);
    },

    async applyOcrResults(
        houseId: string,
        listId: string,
        items: Array<{ name: string; price?: number; quantity?: number; unit?: string }>
    ): Promise<{ detail: string; items: Array<any> }> {
        const token = localStorage.getItem('token');
        const response = await fetch(`${API_URL}/houses/${houseId}/lists/${listId}/ocr/apply`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                Authorization: `Bearer ${token}`,
            },
            body: JSON.stringify(items),
        });
        return handleResponse<{ detail: string; items: Array<any> }>(response);
    },
};
