import { writable } from 'svelte/store';
import type { Chore } from '$lib/types';

interface ChoresState {
    chores: Chore[];
    currentChore: Chore | null;
    isLoading: boolean;
    currentUser: string | null; // User ID for filtering "my chores"
}

function createChoresStore() {
    const { subscribe, set, update } = writable<ChoresState>({
        chores: [],
        currentChore: null,
        isLoading: false,
        currentUser: null
    });

    return {
        subscribe,

        // Set current user ID (for filtering "my chores")
        setCurrentUser: (userId: string) => {
            update(state => ({ ...state, currentUser: userId }));
        },

        // Fetch all chores
        fetchChores: async () => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/chores');
                if (!response.ok) {
                    throw new Error('Failed to fetch chores');
                }

                const chores = await response.json();
                update(state => ({ ...state, chores, isLoading: false }));
                return chores;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Fetch a single chore by ID
        fetchChoreDetails: async (choreId: string) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/chores/${choreId}`);
                if (!response.ok) {
                    throw new Error('Failed to fetch chore details');
                }

                const chore = await response.json();
                update(state => ({ ...state, currentChore: chore, isLoading: false }));
                return chore;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Create a new chore
        createChore: async (choreData: Partial<Chore>) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/chores', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(choreData)
                });

                if (!response.ok) {
                    throw new Error('Failed to create chore');
                }

                const newChore = await response.json();
                update(state => ({
                    ...state,
                    chores: [...state.chores, newChore],
                    currentChore: newChore,
                    isLoading: false
                }));

                return newChore;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Update a chore
        updateChore: async (choreId: string, choreData: Partial<Chore>) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/chores/${choreId}`, {
                    method: 'PUT',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(choreData)
                });

                if (!response.ok) {
                    throw new Error('Failed to update chore');
                }

                const updatedChore = await response.json();
                update(state => ({
                    ...state,
                    chores: state.chores.map(chore =>
                        chore.id === choreId ? updatedChore : chore
                    ),
                    currentChore: state.currentChore?.id === choreId
                        ? updatedChore
                        : state.currentChore,
                    isLoading: false
                }));

                return updatedChore;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Delete a chore
        deleteChore: async (choreId: string) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/chores/${choreId}`, {
                    method: 'DELETE'
                });

                if (!response.ok) {
                    throw new Error('Failed to delete chore');
                }

                update(state => ({
                    ...state,
                    chores: state.chores.filter(chore => chore.id !== choreId),
                    currentChore: state.currentChore?.id === choreId
                        ? null
                        : state.currentChore,
                    isLoading: false
                }));
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Toggle chore completion status
        toggleChoreCompletion: async (choreId: string) => {
            try {
                let currentChore: Chore | null = null as Chore | null;

                // Find the current state of the chore
                update(state => {
                    const chore = state.chores.find(c => c.id === choreId);
                    if (chore) {
                        currentChore = chore;

                        // Optimistically update the UI
                        return {
                            ...state,
                            chores: state.chores.map(c =>
                                c.id === choreId
                                    ? {
                                        ...c,
                                        completed: !c.completed,
                                        completedAt: !c.completed ? new Date().toISOString() : undefined
                                    }
                                    : c
                            ),
                            currentChore: state.currentChore?.id === choreId
                                ? {
                                    ...state.currentChore,
                                    completed: !state.currentChore.completed,
                                    completedAt: !state.currentChore.completed ? new Date().toISOString() : undefined
                                }
                                : state.currentChore
                        };
                    }
                    return state;
                });

                if (!currentChore) {
                    throw new Error('Chore not found');
                }

                // Send the update to the server
                const response = await fetch(`/api/chores/${choreId}`, {
                    method: 'PATCH',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({
                        completed: !currentChore.completed,
                        completedAt: !currentChore.completed ? new Date().toISOString() : null
                    })
                });

                if (!response.ok) {
                    // Revert the optimistic update if the server request fails
                    update(state => ({
                        ...state,
                        chores: state.chores.map(c =>
                            c.id === choreId ? currentChore : c
                        ),
                        currentChore: state.currentChore?.id === choreId
                            ? currentChore
                            : state.currentChore
                    }));

                    throw new Error('Failed to update chore');
                }

                const updatedChore = await response.json();
                return updatedChore;
            } catch (error) {
                throw error;
            }
        },

        // Reassign a chore to a different user
        reassignChore: async (choreId: string, userId: string) => {
            try {
                const response = await fetch(`/api/chores/${choreId}/assign`, {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ userId })
                });

                if (!response.ok) {
                    throw new Error('Failed to reassign chore');
                }

                const updatedChore = await response.json();

                update(state => ({
                    ...state,
                    chores: state.chores.filter(c => c !== null).map(c =>
                        c.id === choreId ? updatedChore : c
                    ) as Chore[],
                    currentChore: state.currentChore?.id === choreId
                        ? updatedChore
                        : state.currentChore
                }));

                return updatedChore;
            } catch (error) {
                throw error;
            }
        }
    };
}

export const choresStore = createChoresStore();
