// src/lib/stores/lists.ts
import { writable } from 'svelte/store';
import type { ShoppingList, ShoppingItem } from '$lib/types';

interface ListsState {
    lists: ShoppingList[];
    currentList: ShoppingList | null;
    isLoading: boolean;
}

function createListsStore() {
    const { subscribe, set, update } = writable<ListsState>({
        lists: [],
        currentList: null,
        isLoading: false
    });

    return {
        subscribe,

        // Fetch all lists
        fetchLists: async () => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/lists');
                if (!response.ok) {
                    throw new Error('Failed to fetch lists');
                }

                const lists = await response.json();
                update(state => ({ ...state, lists, isLoading: false }));
                return lists;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Fetch a single list by ID
        fetchListDetails: async (listId: string) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/lists/${listId}`);
                if (!response.ok) {
                    throw new Error('Failed to fetch list details');
                }

                const list = await response.json();
                update(state => ({ ...state, currentList: list, isLoading: false }));
                return list;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Create a new list
        createList: async (listData: Partial<ShoppingList>) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/lists', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(listData)
                });

                if (!response.ok) {
                    throw new Error('Failed to create list');
                }

                const newList = await response.json();
                update(state => ({
                    ...state,
                    lists: [...state.lists, newList],
                    currentList: newList,
                    isLoading: false
                }));

                return newList;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Update a list
        updateList: async (listId: string, listData: Partial<ShoppingList>) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/lists/${listId}`, {
                    method: 'PUT',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(listData)
                });

                if (!response.ok) {
                    throw new Error('Failed to update list');
                }

                const updatedList = await response.json();
                update(state => ({
                    ...state,
                    lists: state.lists.map(list =>
                        list.id === listId ? updatedList : list
                    ),
                    currentList: state.currentList?.id === listId
                        ? updatedList
                        : state.currentList,
                    isLoading: false
                }));

                return updatedList;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Delete a list
        deleteList: async (listId: string) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch(`/api/lists/${listId}`, {
                    method: 'DELETE'
                });

                if (!response.ok) {
                    throw new Error('Failed to delete list');
                }

                update(state => ({
                    ...state,
                    lists: state.lists.filter(list => list.id !== listId),
                    currentList: state.currentList?.id === listId
                        ? null
                        : state.currentList,
                    isLoading: false
                }));
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Add an item to a list
        addItem: async (listId: string, item: Partial<ShoppingItem>) => {
            try {
                const response = await fetch(`/api/lists/${listId}/items`, {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(item)
                });

                if (!response.ok) {
                    throw new Error('Failed to add item');
                }

                const newItem = await response.json();

                update(state => {
                    // Update the current list if it's the one we're adding to
                    if (state.currentList && state.currentList.id === listId) {
                        return {
                            ...state,
                            currentList: {
                                ...state.currentList,
                                items: [...state.currentList.items, newItem],
                                updatedAt: new Date().toISOString()
                            }
                        };
                    }

                    // Update the list in the lists array
                    return {
                        ...state,
                        lists: state.lists.map(list => {
                            if (list.id === listId) {
                                return {
                                    ...list,
                                    items: [...(list.items || []), newItem],
                                    updatedAt: new Date().toISOString()
                                };
                            }
                            return list;
                        })
                    };
                });

                return newItem;
            } catch (error) {
                throw error;
            }
        },

        // Add multiple items to a list
        addMultipleItems: async (listId: string, items: Partial<ShoppingItem>[]) => {
            try {
                const response = await fetch(`/api/lists/${listId}/items/batch`, {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ items })
                });

                if (!response.ok) {
                    throw new Error('Failed to add items');
                }

                const newItems = await response.json();

                update(state => {
                    // Update the current list if it's the one we're adding to
                    if (state.currentList && state.currentList.id === listId) {
                        return {
                            ...state,
                            currentList: {
                                ...state.currentList,
                                items: [...state.currentList.items, ...newItems],
                                updatedAt: new Date().toISOString()
                            }
                        };
                    }

                    // Update the list in the lists array
                    return {
                        ...state,
                        lists: state.lists.map(list => {
                            if (list.id === listId) {
                                return {
                                    ...list,
                                    items: [...(list.items || []), ...newItems],
                                    updatedAt: new Date().toISOString()
                                };
                            }
                            return list;
                        })
                    };
                });

                return newItems;
            } catch (error) {
                throw error;
            }
        },

        // Toggle item completion status
        toggleItemCompletion: async (listId: string, itemId: string) => {
            try {
                let currentItem;

                // Find the current state of the item
                update(state => {
                    if (state.currentList && state.currentList.id === listId) {
                        const item = state.currentList.items.find(i => i.id === itemId);
                        if (item) {
                            currentItem = item;

                            // Optimistically update the UI
                            return {
                                ...state,
                                currentList: {
                                    ...state.currentList,
                                    items: state.currentList.items.map(i =>
                                        i.id === itemId ? { ...i, completed: !i.completed } : i
                                    ),
                                    updatedAt: new Date().toISOString()
                                }
                            };
                        }
                    }
                    return state;
                });

                if (!currentItem) {
                    throw new Error('Item not found');
                }

                // Send the update to the server
                const response = await fetch(`/api/lists/${listId}/items/${itemId}`, {
                    method: 'PATCH',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ completed: !currentItem.completed })
                });

                if (!response.ok) {
                    // Revert the optimistic update if the server request fails
                    update(state => {
                        if (state.currentList && state.currentList.id === listId) {
                            return {
                                ...state,
                                currentList: {
                                    ...state.currentList,
                                    items: state.currentList.items.map(i =>
                                        i.id === itemId ? { ...i, completed: currentItem.completed } : i
                                    )
                                }
                            };
                        }
                        return state;
                    });

                    throw new Error('Failed to update item');
                }

                const updatedItem = await response.json();
                return updatedItem;
            } catch (error) {
                throw error;
            }
        },

        // Delete an item from a list
        deleteItem: async (listId: string, itemId: string) => {
            try {
                // Optimistically update the UI
                update(state => {
                    if (state.currentList && state.currentList.id === listId) {
                        return {
                            ...state,
                            currentList: {
                                ...state.currentList,
                                items: state.currentList.items.filter(i => i.id !== itemId),
                                updatedAt: new Date().toISOString()
                            }
                        };
                    }

                    return {
                        ...state,
                        lists: state.lists.map(list => {
                            if (list.id === listId) {
                                return {
                                    ...list,
                                    items: (list.items || []).filter(i => i.id !== itemId),
                                    updatedAt: new Date().toISOString()
                                };
                            }
                            return list;
                        })
                    };
                });

                // Send the delete request to the server
                const response = await fetch(`/api/lists/${listId}/items/${itemId}`, {
                    method: 'DELETE'
                });

                if (!response.ok) {
                    // If the server request fails, we should fetch the list again to get the correct state
                    await this.fetchListDetails(listId);
                    throw new Error('Failed to delete item');
                }
            } catch (error) {
                throw error;
            }
        }
    };
}

export const listsStore = createListsStore();
