import type { User } from '$lib/types';
import { writable } from 'svelte/store';

interface UserState {
    user: User | null;
    isLoggedIn: boolean;
    isLoading: boolean;
    avatar?: string;
}

function createUserStore() {
    const { subscribe, set, update } = writable<UserState>({
        user: null,
        isLoggedIn: false,
        isLoading: true
    });

    return {
        subscribe,

        // Initialize the store and check for existing session
        init: async () => {
            try {
                const response = await fetch('/api/auth/me');
                if (response.ok) {
                    const user = await response.json();
                    set({ user, isLoggedIn: true, isLoading: false });
                } else {
                    set({ user: null, isLoggedIn: false, isLoading: false });
                }
            } catch (error) {
                console.error('Failed to initialize user session:', error);
                set({ user: null, isLoggedIn: false, isLoading: false });
            }
        },

        // Login user
        login: async (email: string, password: string, rememberMe: boolean = false) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/auth/login', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ email, password, rememberMe })
                });

                if (!response.ok) {
                    const error = await response.json();
                    throw new Error(error.message || 'Login failed');
                }

                const user = await response.json();
                set({ user, isLoggedIn: true, isLoading: false });
                return user;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Register new user
        register: async (name: string, email: string, password: string) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/auth/register', {
                    method: 'POST',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify({ name, email, password })
                });

                if (!response.ok) {
                    const error = await response.json();
                    throw new Error(error.message || 'Registration failed');
                }

                const user = await response.json();
                set({ user, isLoggedIn: true, isLoading: false });
                return user;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Logout user
        logout: async () => {
            try {
                await fetch('/api/auth/logout', { method: 'POST' });
            } catch (error) {
                console.error('Logout error:', error);
            } finally {
                set({ user: null, isLoggedIn: false, isLoading: false });
            }
        },

        // Update user profile
        updateProfile: async (userData: Partial<User>) => {
            update(state => ({ ...state, isLoading: true }));

            try {
                const response = await fetch('/api/auth/profile', {
                    method: 'PUT',
                    headers: { 'Content-Type': 'application/json' },
                    body: JSON.stringify(userData)
                });

                if (!response.ok) {
                    const error = await response.json();
                    throw new Error(error.message || 'Failed to update profile');
                }

                const updatedUser = await response.json();
                update(state => ({
                    ...state,
                    user: { ...state.user, ...updatedUser },
                    isLoading: false
                }));

                return updatedUser;
            } catch (error) {
                update(state => ({ ...state, isLoading: false }));
                throw error;
            }
        },

        // Fetch users
        getUsers: async (): Promise<User[]> => {
            try {
                const response = await fetch('/api/users');
                if (!response.ok) {
                    throw new Error('Failed to fetch users');
                }
                return await response.json();
            } catch (error) {
                console.error('Failed to fetch users:', error);
                throw error;
            }
        }
    };
}

export const userStore = createUserStore();
